#ifndef RANDOMPW_HEADER_DEFINE
#define RANDOMPW_HEADER_DEFINED

/*-------------------------------------------------------------------*/
/* randompw.h                                                        */
/* ----------                                                        */
/*                                                                   */
/* Header file for "randompw".  "randompw" is a program that will    */
/* generate random passwords.  These password can be between four    */
/* and thirty two bytes (characters) in length.  Depending on the    */
/* options chosen, "randompw" will generate passwords using upper    */
/* case, lower case, numbers and symbols or any combination of the   */
/* latter.                                                           */
/*                                                                   */
/* "randompw" can be used within a "useradd" script to automatically */
/* generate passwords whenever a new user is defined.                */
/*                                                                   */
/* Copyright (c) 1987-2005                                           */
/* Marc Niegowski                                                    */
/* Connectivity, Inc.                                                */
/* All rights reserved.                                              */
/*                                                                   */
/* 23 W. Fourth Street                                               */
/* Media, PA 19063-2805                                              */
/* USA                                                               */
/*                                                                   */
/* Phone: (610) 566-0227                                             */
/* Fax:   (610) 566-0641                                             */
/*                                                                   */
/* Web:   http://www.marcsweb.com                                    */
/* FTP:   ftp://ftp.marcsweb.com                                     */
/*                                                                   */
/* Email: Marc@Tech-Center.com                                       */
/*-------------------------------------------------------------------*/

#define RPW_VERSION "1.2.0"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE !FALSE
#endif

#define RPW_NO_ERROR       0
#define RPW_TIME_ERROR    -1
#define RPW_PARM_ERROR    -2

#define RPW_MAX_LENGTH    32
#define RPW_MIN_LENGTH     4
#define RPW_DEFAULT_LENGTH 8

#define RPW_MAX_COUNT  32767     
#define RPW_MIN_COUNT      1
#define RPW_DEFAULT_COUNT  1

                               // -- Bits --
#define RPW_CLEAR_FLAG     0   // ....  ....
#define RPW_UPPER_FLAG     1   // ....  ...1
#define RPW_LOWER_FLAG     2   // ....  ..1.
#define RPW_ALPHA_FLAG     4   // ....  .1..
#define RPW_SYMBOL_FLAG    8   // ....  1...
#define RPW_SEQUENCE_FLAG 16   // ...1  ....
#define RPW_NUMBER_FLAG   32   // ..1.  ....
#define RPW_HASALPHA_FLAG 64   // .1..  ....
#define RPW_HASNUM_FLAG  128   // 1...  ....

typedef
struct
{
    int    retcode;
    int    pwlen;
    int    pwcount;
    char   flags;
    char  *message;
}   rpw_struct;

#ifdef _WIN32
#define timeval _timeb
#endif

typedef
struct
timeval rpw_tv;

static
const
char *rpw_header_str  = "\nrandompw Version " RPW_VERSION "\n"
                        "Built on " __DATE__ " at " __TIME__ "\n\n";

static
const
char *rpw_error_str   = "Error: Invalid parameter(s).\n"
                        "Use -h or --help for help.\n\n";

static
const
char *rpw_logic_str   = 

"Error: Logical parameter error.\n"
"You cannot combine -u (uppercase only) with -l (lowercase only).\n\n";

static
const
char *rpw_rules_str   =

"Error: Logical rules parameter error.\n"
"You cannot combine -r (require at least one number) with\n"
"-a (alphabetic only).\n\n";
 
static
const
char *rpw_version_str = "";

static
const
char *rpw_help_str    =

"Random Password Generator\n"
"Copyright (c) 1987-2005, Marc Niegowski\n"
"Connectivity, Inc.\n"
"All rights reserved.\n\n"
"Usage: randompw [OPTIONS]\n\n"
"Options: [-h] [-v] [-s<nn>] [-n<nnnnn>] [-u | -l] [-a | -r] [-y] [-q]\n\n"
"-h                Display this help information.\n"
"--help            Any other options are ignored.\n\n"
"-v                Displays the randompw version.\n"
"--version\n\n"
"-s<4-32>          Sets the size of the password to\n"
"--size=<4-32>     generate (Default is 8).  The size must be\n"
"--size:<4-32>     between 4 and 32 characters.\n"
"                  (e.g., -s10 or --size:10)\n\n"
"-n<1-32767>       The number of passwords to generate.\n"
"--number=<1-32767>\n"
"--number:<1-32767>\n\n"
"-u                Produces passwords containing only\n"
"--upper           upper case characters.  This option may\n"
"--upper-only      be combined with the \"alpha\" and/or the\n"
"--uppercase       \"symbol\" options.  Obviously, it can not\n"
"--uppercase-only  be combined with the \"lower\" option.\n"
"--upper-case\n"
"--upper-case-only\n\n"
"-l                Produces passwords containing only\n"
"--lower           lower case characters.  This option may\n"
"--lower-only      be combined with the \"alpha\" and/or the\n"
"--lowercase       \"symbol\" options.  Obviously, it can not\n"
"--lowercase-only  be combined with the \"upper\" option.\n"
"--lower-case\n"
"--lower-case-only\n\n"
"-a                Passwords are generated using only alphabetic\n"
"--alpha-only      characters and no numerics.  If used in conjunction\n"
"--alphabetic      with the \"symbol\" option, passwords are generated\n"
"--alphabetic-only with alphabetic characters and symbols.  This\n"
"                  option may be combined with the \"upper\" or the\n"
"                  \"lower\" option.  Obviously, it can not be combined\n"
"                  with the \"numeric-required\" option.\n\n"
"-r                Generated passwords must contain at least one number.\n"
"--required        This option is obviously mutually exclusive with the\n"
"--required-number \"alpha-only\" option.\n"
"--required-numeric\n\n"
"-y                Includes symbol characters in the generated\n"
"--symbols         password.\n"
"--include-symbols\n\n"
"-q                Generate passwords that contain no sequences.\n"
"--sequence        Sequences are two or more of the same characters\n"
"--sequence-check  or sequential characters in a row\n"
"                  (e.g., \"cc\", \"bc\", \"cb\", \"34\", \"44\", \"43\")\n\n";

static
const
char *rpw_parm[][2]   =
{
    {"help",            "h"},
    {"version",         "v"},
    {"size",            "s"},
    {"number",          "n"},
    {"upper",           "u"},
    {"upper-only",      "u"},
    {"uppercase",       "u"},
    {"upper-case",      "u"},
    {"uppercase-only",  "u"},
    {"upper-case-only", "u"},
    {"lower",           "l"},
    {"lower-only",      "l"},
    {"lowercase",       "l"},
    {"lower-case",      "l"},
    {"lowercase-only",  "l"},
    {"lower-case-only", "l"},
    {"alpha",           "a"},
    {"alpha-only",      "a"},
    {"alphabetic",      "a"},
    {"alphabetic-only", "a"},
    {"required",        "r"},
    {"required-number", "r"},
    {"required-numeric","r"},
    {"symbols",         "y"},
    {"include_symbols", "y"},
    {"sequence",        "q"},
    {"sequence-check",  "q"}
};
    
static
const
char *rpw_pwupper     = "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
     *rpw_pwlower     = "abcdefghijklmnopqrstuvwxyz",
     *rpw_pwnumbers   = "0123456789",
     *rpw_pwsymbols   = "!#$%&*+-.:=?@^_|";

#define RPW_ALPHA_SIZE  26
#define RPW_NUMBER_SIZE 10
#define RPW_SYMBOL_SIZE 16

static
int   rpw_return(rpw_struct *);
static
void  rpw_getargs(rpw_struct *, int, char **);
static
int   rpw_short_parm(rpw_struct *, char *);
static
int   rpw_long_parm(rpw_struct *, char *);
static
int   rpw_check_limit(char *, int, int);
static
int   rpw_parm_error(rpw_struct *);
static
int   rpw_generate(rpw_struct *);
static
void  rpw_inject_char(char *, char *);

#endif
